@@ -1,13 +1,13 @@
 package CGI::Session;
 
-# $Id: Session.pm 299 2006-04-12 20:56:26Z antirice $
+# $Id: Session.pm 407 2008-04-08 00:56:47Z ron $
 
 use strict;
 use Carp;
 use CGI::Session::ErrorHandler;
 
 @CGI::Session::ISA      = qw( CGI::Session::ErrorHandler );
-$CGI::Session::VERSION  = '4.13';
+$CGI::Session::VERSION  = '4.30';
 $CGI::Session::NAME     = 'CGISESSID';
 $CGI::Session::IP_MATCH = 0;
 
@@ -17,49 +17,71 @@ sub STATUS_DELETED  () { 4 }        # denotes session that needs deletion
 sub STATUS_EXPIRED  () { 8 }        # denotes session that was expired.
 
 sub import {
-    my $class = shift;
-    @_ or return;
+    my ($class, @args) = @_;
 
-    for(@_) {
-        $CGI::Session::IP_MATCH = ( $_ eq '-ip_match' );
+    return unless @args;
+
+  ARG:
+    foreach my $arg (@args) {
+        if ($arg eq '-ip_match') {
+            $CGI::Session::IP_MATCH = 1;
+            last ARG;
+        }
     }
 }
 
 sub new {
-    my $class   = shift;
+    my ($class, @args) = @_;
 
-    # If called as object method as in $session->new()...
     my $self;
-    if ( ref $class ) {
-        $self   = bless { %$class }, ref($class);
-        $class  = ref($class);
+    if (ref $class) {
+        #
+        # Called as an object method as in $session->new()...
+        #
+        $self  = bless { %$class }, ref( $class );
+        $class = ref $class;
         $self->_reset_status();
-
-        # Object may still have public data associated with it, but we don't care about that,
-        # since we want to leave that to the client's disposal. However, if new() was requested on
-        # an expired session, we already know that '_DATA' table is empty, since it was the
-        # job of flush() to empty '_DATA' after deleting. How do we know flush() was already
-        # called on an expired session? Because load() - constructor always calls flush()
-        # on all to-be expired sessions
-    } else {
-        defined($self = $class->load( @_ ))
-            or return $class->set_error( "new(): failed: " . $class->errstr );
+        #
+        # Object may still have public data associated with it, but we
+        # don't care about that, since we want to leave that to the
+        # client's disposal. However, if new() was requested on an
+        # expired session, we already know that '_DATA' table is
+        # empty, since it was the job of flush() to empty '_DATA'
+        # after deleting. How do we know flush() was already called on
+        # an expired session? Because load() - constructor always
+        # calls flush() on all to-be expired sessions
+        #
     }
-
-    # Absence of '_SESSION_ID' can only signal:
-    #   * expired session
-    #       Because load() - constructor is required to empty contents of _DATA - table
-    #   * unavailable session
-    #       Such sessions are the ones that don't exist on datastore, but requested by client
-    #   * new sessions
-    #       When no specific session is requested to be loaded
-    unless ( $self->{_DATA}->{_SESSION_ID} ) {
-        $self->{_DATA}->{_SESSION_ID} = $self->_id_generator()->generate_id($self->{_DRIVER_ARGS}, $self->{_CLAIMED_ID});
-        unless ( defined $self->{_DATA}->{_SESSION_ID} ) {
-            return $self->set_error( "Couldn't generate new SID" );
+    else {
+        #
+        # Called as a class method as in CGI::Session->new()
+        #
+        $self = $class->load( @args );
+        if (not defined $self) {
+            return $class->set_error( "new(): failed: " . $class->errstr );
         }
-        $self->{_DATA}->{_SESSION_CTIME} = $self->{_DATA}->{_SESSION_ATIME} = time();
-        $self->_set_status(STATUS_NEW);
+    }
+    my $dataref = $self->{_DATA};
+    unless ($dataref->{_SESSION_ID}) {
+        #
+        # Absence of '_SESSION_ID' can only signal:
+        # * Expired session: Because load() - constructor is required to
+        #                    empty contents of _DATA - table
+        # * Unavailable session: Such sessions are the ones that don't
+        #                    exist on datastore, but are requested by client
+        # * New session: When no specific session is requested to be loaded
+        #
+        my $id = $self->_id_generator()->generate_id(
+                                                     $self->{_DRIVER_ARGS},
+                                                     $self->{_CLAIMED_ID}
+                                                     );
+        unless (defined $id) {
+            return $self->set_error( "Couldn't generate new SESSION-ID" );
+        }
+        $dataref->{_SESSION_ID} = $id;
+        $dataref->{_SESSION_CTIME} = $dataref->{_SESSION_ATIME} = time();
+        $dataref->{_SESSION_REMOTE_ADDR} = $ENV{REMOTE_ADDR} || "";
+        $self->_set_status( STATUS_NEW );
     }
     return $self;
 }
@@ -79,8 +101,10 @@ sub is_new          { $_[0]->_test_status( STATUS_NEW ) }
 
 sub id              { return defined($_[0]->dataref) ? $_[0]->dataref->{_SESSION_ID}    : undef }
 
+# Last Access Time
 sub atime           { return defined($_[0]->dataref) ? $_[0]->dataref->{_SESSION_ATIME} : undef }
 
+# Creation Time
 sub ctime           { return defined($_[0]->dataref) ? $_[0]->dataref->{_SESSION_CTIME} : undef }
 
 sub _driver {
@@ -189,6 +213,11 @@ sub _test_status {
 sub flush {
     my $self = shift;
 
+    # Would it be better to die or err if something very basic is wrong here? 
+    # I'm trying to address the DESTORY related warning
+    # from: http://rt.cpan.org/Ticket/Display.html?id=17541
+    # return unless defined $self;
+
     return unless $self->id;            # <-- empty session
     return if !defined($self->{_STATUS}) or $self->{_STATUS} == 0;    # <-- neither new, nor deleted nor modified
 
@@ -224,42 +253,39 @@ sub trace {}
 sub tracemsg {}
 
 sub param {
-    my $self = shift;
+    my ($self, @args) = @_;
 
-    carp "param(): attempt to read/write deleted session" if $self->_test_status(STATUS_DELETED);
-
-    #
-    # USAGE: $s->param();
-    # DESC: returns all the **public** parameters
-    unless ( @_ ) {
-        return grep { !/^_SESSION_/ } keys %{ $self->{_DATA} };
+    if ($self->_test_status( STATUS_DELETED )) {
+        carp "param(): attempt to read/write deleted session";
     }
 
-    #
-    # USAGE: $s->param($p);
+    # USAGE: $s->param();
+    # DESC:  Returns all the /public/ parameters
+    if (@args == 0) {
+        return grep { !/^_SESSION_/ } keys %{ $self->{_DATA} };
+    }
+    # USAGE: $s->param( $p );
     # DESC: returns a specific session parameter
-    return $self->{_DATA}->{$_[0]} if @_ == 1;
+    elsif (@args == 1) {
+        return $self->{_DATA}->{ $args[0] }
+    }
 
-    my %args = (
-        -name   => undef,
-        -value  => undef,
-        @_
-    );
 
-    #
-    # USAGE: $s->param(-name=>$n, -value=>$v);
-    # DESC:  updates session data using CGI.pm's 'named parameter' syntax. Only
-    # public records can be set!
-    if ( defined( $args{'-name'} ) && defined( $args{'-value'} ) ) {
-        if ( $args{'-name'} =~ m/^_SESSION_/ ) {
+    # USAGE: $s->param( -name => $n, -value => $v );
+    # DESC:  Updates session data using CGI.pm's 'named param' syntax.
+    #        Only public records can be set!
+    my %args = @args;
+    my ($name, $value) = @args{ qw(-name -value) };
+    if (defined $name && defined $value) {
+        if ($name =~ m/^_SESSION_/) {
+
             carp "param(): attempt to write to private parameter";
             return undef;
         }
-        $self->_set_status(STATUS_MODIFIED);
-        return $self->{_DATA}->{ $args{'-name'} } = $args{'-value'};
+        $self->_set_status( STATUS_MODIFIED );
+        return $self->{_DATA}->{ $name } = $value;
     }
 
-    #
     # USAGE: $s->param(-name=>$n);
     # DESC:  access to session data (public & private) using CGI.pm's 'named parameter' syntax.
     return $self->{_DATA}->{ $args{'-name'} } if defined $args{'-name'};
@@ -267,21 +293,24 @@ sub param {
     # USAGE: $s->param($name, $value);
     # USAGE: $s->param($name1 => $value1, $name2 => $value2 [,...]);
     # DESC:  updates one or more **public** records using simple syntax
-    unless ( @_ % 2 ) {
-        for ( my $i=0; $i < @_; $i += 2 ) {
-            if ( $_[$i] =~ m/^_SESSION_/) {
+    if ((@args % 2) == 0) {
+        my $modified_cnt = 0;
+	ARG_PAIR:
+        while (my ($name, $val) = each %args) {
+            if ( $name =~ m/^_SESSION_/) {
                 carp "param(): attempt to write to private parameter";
-                next;
+                next ARG_PAIR;
             }
-            $self->{_DATA}->{ $_[$i] } = $_[$i+1];
+            $self->{_DATA}->{ $name } = $val;
+            ++$modified_cnt;
         }
         $self->_set_status(STATUS_MODIFIED);
-        return 1;
+        return $modified_cnt;
     }
 
-    #
-    # If we reached this far none of the expected syntax were detected. Syntax error
-    croak "param(): usage error. Invalid number";
+    # If we reached this far none of the expected syntax were
+    # detected. Syntax error
+    croak "param(): usage error. Invalid syntax";
 }
 
 
@@ -362,7 +391,7 @@ sub clear {
         $params = [ $self->param ];
     }
 
-    for ( @$params ) {
+    for ( grep { ! /^_SESSION_/ } @$params ) {
         delete $self->{_DATA}->{$_};
     }
     $self->_set_status( STATUS_MODIFIED );
@@ -371,12 +400,15 @@ sub clear {
 
 sub find {
     my $class       = shift;
-    my ($dsnstr, $coderef, $dsn_args);
+    my ($dsn, $coderef, $dsn_args);
 
+    # find( \%code )
     if ( @_ == 1 ) {
         $coderef = $_[0];
-    } else {
-        ($dsnstr, $coderef, $dsn_args) = @_;
+    } 
+    # find( $dsn, \&code, \%dsn_args )
+    else {
+        ($dsn, $coderef, $dsn_args) = @_;
     }
 
     unless ( $coderef && ref($coderef) && (ref $coderef eq 'CODE') ) {
@@ -384,12 +416,12 @@ sub find {
     }
 
     my $driver;
-    if ( $dsnstr ) {
-        my $hashref = $class->parse_dsn( $dsnstr );
+    if ( $dsn ) {
+        my $hashref = $class->parse_dsn( $dsn );
         $driver     = $hashref->{driver};
     }
     $driver ||= "file";
-    my $pm = "CGI::Session::Driver::" . $driver;
+    my $pm = "CGI::Session::Driver::" . ($driver =~ /(.*)/)[0];
     eval "require $pm";
     if (my $errmsg = $@ ) {
         return $class->set_error( "find(): couldn't load driver." . $errmsg );
@@ -400,9 +432,10 @@ sub find {
         return $class->set_error( "find(): couldn't create driver object. " . $pm->errstr );
     }
 
+    my $dont_update_atime = 0;
     my $driver_coderef = sub {
         my ($sid) = @_;
-        my $session = $class->load( $dsnstr, $sid, $dsn_args );
+        my $session = $class->load( $dsn, $sid, $dsn_args, $dont_update_atime );
         unless ( $session ) {
             return $class->set_error( "find(): couldn't load session '$sid'. " . $class->errstr );
         }
@@ -414,7 +447,7 @@ sub find {
     return 1;
 }
 
-# $Id: Session.pm 299 2006-04-12 20:56:26Z antirice $
+# $Id: Session.pm 407 2008-04-08 00:56:47Z ron $
 
 =pod
 
@@ -438,6 +471,10 @@ CGI::Session - persistent session data in CGI applications
     # or
     $session->param(-name=>'l_name', -value=>'Ruzmetov');
 
+    # flush the data from memory to the storage driver at least before your
+    # program finishes since auto-flushing can be unreliable
+    $session->flush();
+
     # retrieving data
     my $f_name = $session->param('f_name');
     # or
@@ -461,6 +498,42 @@ CGI-Session is a Perl5 library that provides an easy, reliable and modular sessi
 Persistency is a key feature for such applications as shopping carts, login/authentication routines, and application that
 need to carry data across HTTP requests. CGI::Session does that and many more.
 
+=head1 A Warning about Auto-flushing
+
+As mentioned above in the Synopsis, auto-flushing can be unreliable.
+
+Consequently, you should regard it as mandatory that sessions always need to be explicitly flushed before the
+program exits.
+
+For instance, in a C<CGI::Application>-based program, C<sub teardown()> would be the appropriate place to do this.
+
+=head1 A Warning about UTF8
+
+Trying to use UTF8 in a program which uses CGI::Session has lead to problems. See RT#21981 and RT#28516.
+
+In the first case the user tried "use encoding 'utf8';" in the program, and in the second case the user tried
+"$dbh->do(qq|set names 'utf8'|);".
+
+Until this problem is understood and corrected, users are advised to avoid UTF8 in conjunction with CGI::Session.
+
+For details, see: http://rt.cpan.org/Public/Bug/Display.html?id=28516 (and ...id=21981).
+
+=head1 TRANSLATIONS
+
+This document is also available in Japanese.
+
+=over 4
+
+=item o 
+
+Translation based on 4.14: http://digit.que.ne.jp/work/index.cgi?Perldoc/ja
+
+=item o
+
+Translation based on 3.11, including Cookbook and Tutorial: http://perldoc.jp/docs/modules/CGI-Session-3.11/
+
+=back
+
 =head1 TO LEARN MORE
 
 Current manual is optimized to be used as a quick reference. To learn more both about the philosophy and CGI::Session
@@ -494,17 +567,17 @@ L<Apache::Session|Apache::Session> - another fine alternative to CGI::Session.
 
 Following is the overview of all the available methods accessible via CGI::Session object.
 
-=over 4
+=head2 new()
 
-=item new()
+=head2 new( $sid )
 
-=item new( $sid )
+=head2 new( $query )
 
-=item new( $query )
+=head2 new( $dsn, $query||$sid )
 
-=item new( $dsn, $query||$sid )
+=head2 new( $dsn, $query||$sid, \%dsn_args )
 
-=item new( $dsn, $query||$sid, \%dsn_args )
+=head2 new( $dsn, $query||$sid, \%dsn_args, \%session_params )
 
 Constructor. Returns new session object, or undef on failure. Error message is accessible through L<errstr() - class method|CGI::Session::ErrorHandler/errstr>. If called on an already initialized session will re-initialize the session based on already configured object. This is only useful after a call to L<load()|/"load">.
 
@@ -522,6 +595,14 @@ If called with two arguments first will be treated as $dsn, and second will be t
     $s = CGI::Session->new("serializer:storable;id:incr", $sid);
     # etc...
 
+Briefly, C<new()> will return an initialized session object with a valid id, whereas C<load()> may return
+an empty session object with an undefined id.
+
+Tests are provided (t/new_with_undef.t and t/load_with_undef.t) to clarify the result of calling C<new()> and C<load()>
+with undef, or with an initialized CGI object with an undefined or fake CGISESSID.
+
+You are strongly advised to run the old-fashioned 'make test TEST_FILES=t/new_with_undef.t TEST_VERBOSE=1'
+or the new-fangled 'prove -v t/new_with_undef.t', for both new*.t and load*.t, and examine the output.
 
 Following data source components are supported:
 
@@ -548,17 +629,33 @@ For example, to get CGI::Session store its data using DB_File and serialize data
 
 If called with three arguments, first two will be treated as in the previous example, and third argument will be C<\%dsn_args>, which will be passed to C<$dsn> components (namely, driver, serializer and id generators) for initialization purposes. Since all the $dsn components must initialize to some default value, this third argument should not be required for most drivers to operate properly.
 
+If called with four arguments, the first three match previous examples. The fourth argument must be a hash reference with parameters to be used by the CGI::Session object. (see \%session_params above )
+
+The following is a list of the current keys:
+
+=over
+
+=item *
+
+B<name> - Name to use for the cookie/query parameter name. This defaults to CGISESSID. This can be altered or accessed by the C<name> accessor.
+
+=back
+
 undef is acceptable as a valid placeholder to any of the above arguments, which will force default behavior.
 
-=item load()
+=head2 load()
 
-=item load($query||$sid)
+=head2 load( $query||$sid )
 
-=item load($dsn, $query||$sid)
+=head2 load( $dsn, $query||$sid )
 
-=item load($dsn, $query, \%dsn_args);
+=head2 load( $dsn, $query, \%dsn_args )
 
-Constructor. Usage is identical to L<new()|/"new">, so is the return value. Major difference is, L<new()|/"new"> can create new session if it detects expired and non-existing sessions, but C<load()> does not.
+=head2 load( $dsn, $query, \%dsn_args, \%session_params )
+
+Accepts the same arguments as new(), and also returns a new session object, or
+undef on failure.  The difference is, L<new()|/"new"> can create a new session if
+it detects expired and non-existing sessions, but C<load()> does not.
 
 C<load()> is useful to detect expired or non-existing sessions without forcing the library to create new sessions. So now you can do something like this:
 
@@ -575,15 +672,26 @@ C<load()> is useful to detect expired or non-existing sessions without forcing t
         $s = $s->new() or die $s->errstr;
     }
 
-Notice, all I<expired> sessions are empty, but not all I<empty> sessions are expired!
+Notice: All I<expired> sessions are empty, but not all I<empty> sessions are expired!
+
+Briefly, C<new()> will return an initialized session object with a valid id, whereas C<load()> may return
+an empty session object with an undefined id.
+
+Tests are provided (t/new_with_undef.t and t/load_with_undef.t) to clarify the result of calling C<new()> and C<load()>
+with undef, or with an initialized CGI object with an undefined or fake CGISESSID.
+
+You are strongly advised to run the old-fashioned 'make test TEST_FILES=t/new_with_undef.t TEST_VERBOSE=1'
+or the new-fangled 'prove -v t/new_with_undef.t', for both new*.t and load*.t, and examine the output.
 
 =cut
 
+# pass a true value as the fourth parameter if you want to skip the changing of
+# access time This isn't documented more formally, because it only called by
+# find().
 sub load {
     my $class = shift;
-
     return $class->set_error( "called as instance method")    if ref $class;
-    return $class->set_error( "invalid number of arguments")  if @_ > 3;
+    return $class->set_error( "Too many arguments")  if @_ > 5;
 
     my $self = bless {
         _DATA       => {
@@ -607,53 +715,48 @@ sub load {
         _QUERY      => undef        # query object
     }, $class;
 
-    #$self->{_DATA}->{_SESSION_CTIME} = $self->{_DATA}->{_SESSION_ATIME} = time();
-
+    my ($dsn,$query_or_sid,$dsn_args,$update_atime,$params);
+    # load($query||$sid)
     if ( @_ == 1 ) {
-        if ( ref $_[0] ){ $self->{_QUERY}       = $_[0]  }
-        else            { $self->{_CLAIMED_ID}  = $_[0]  }
+        $self->_set_query_or_sid($_[0]);
     }
-
     # Two or more args passed:
-    if ( @_ > 1 ) {
-        if ( defined $_[0] ) {      # <-- to avoid 'Uninitialized value...' warnings
-            $self->{_DSN} = $self->parse_dsn( $_[0] );
+    # load($dsn, $query||$sid)
+    elsif ( @_ > 1 ) {
+        ($dsn, $query_or_sid, $dsn_args,$update_atime) = @_;
+
+        # Make it backwards-compatible (update_atime is an undocumented key in %$params).
+        # In fact, update_atime as a key is not used anywhere in the code as yet.
+        # This patch is part of the patch for RT#33437.
+        if ( ref $update_atime and ref $update_atime eq 'HASH' ) {
+            $params = {%$update_atime};
+            $update_atime = $params->{'update_atime'};
+
+            if ($params->{'name'}) {
+                $self->{_NAME} = $params->{'name'};
+            }
         }
-        #
-        # second argument can either be $sid, or  $query
-        if ( ref $_[1] ){ $self->{_QUERY}       = $_[1] }
-        else            { $self->{_CLAIMED_ID}  = $_[1] }
+
+        # Since $update_atime is not part of the public API
+        # we ignore any value but the one we use internally: 0.
+        if (defined $update_atime and $update_atime ne '0') {
+            return $class->set_error( "Too many arguments");
+         }
+
+        if ( defined $dsn ) {      # <-- to avoid 'Uninitialized value...' warnings
+            $self->{_DSN} = $self->parse_dsn($dsn);
+        }
+        $self->_set_query_or_sid($query_or_sid);
+
+        # load($dsn, $query, \%dsn_args);
+
+        $self->{_DRIVER_ARGS} = $dsn_args if defined $dsn_args;
+
     }
 
-    #
-    # grabbing the 3rd argument, if any
-    if ( @_ == 3 ){ $self->{_DRIVER_ARGS} = $_[2] }
+    $self->_load_pluggables();
 
-    #
-    # setting defaults, since above arguments might be 'undef'
-    $self->{_DSN}->{driver}     ||= "file";
-    $self->{_DSN}->{serializer} ||= "default";
-    $self->{_DSN}->{id}         ||= "md5";
-
-    # Beyond this point used to be '_init()' method. But I had to merge them together
-    # since '_init()' did not serve specific purpose
-
-
-    #
-    # Checking and loading driver, serializer and id-generators
-    #
-    my @pms = ();
-    $pms[0] = "CGI::Session::Driver::"      . $self->{_DSN}->{driver};
-    $pms[1] = "CGI::Session::Serialize::"  . $self->{_DSN}->{serializer};
-    $pms[2] = "CGI::Session::ID::"          . $self->{_DSN}->{id};
-    for ( @pms ) {
-        eval "require $_";
-        if ( my $errmsg = $@ ) {
-            return $self->set_error("couldn't load $_: " . $errmsg);
-        }
-    }
-
-    unless ( $self->{_CLAIMED_ID} ) {
+    if (not defined $self->{_CLAIMED_ID}) {
         my $query = $self->query();
         eval {
             $self->{_CLAIMED_ID} = $query->cookie( $self->name ) || $query->param( $self->name );
@@ -663,18 +766,16 @@ sub load {
         }
     }
 
-    #
     # No session is being requested. Just return an empty session
     return $self unless $self->{_CLAIMED_ID};
 
-    #
     # Attempting to load the session
     my $driver = $self->_driver();
     my $raw_data = $driver->retrieve( $self->{_CLAIMED_ID} );
     unless ( defined $raw_data ) {
         return $self->set_error( "load(): couldn't retrieve data: " . $driver->errstr );
     }
-    #
+    
     # Requested session couldn't be retrieved
     return $self unless $raw_data;
 
@@ -690,7 +791,6 @@ sub load {
         return $self->set_error( "Invalid data structure returned from thaw()" );
     }
 
-    #
     # checking if previous session ip matches current ip
     if($CGI::Session::IP_MATCH) {
       unless($self->_ip_matches) {
@@ -700,7 +800,6 @@ sub load {
       }
     }
 
-    #
     # checking for expiration ticker
     if ( $self->{_DATA}->{_SESSION_ETIME} ) {
         if ( ($self->{_DATA}->{_SESSION_ATIME} + $self->{_DATA}->{_SESSION_ETIME}) <= time() ) {
@@ -719,27 +818,85 @@ sub load {
         }
     }
     $self->clear(\@expired_params) if @expired_params;
-    $self->{_DATA}->{_SESSION_ATIME} = time();      # <-- updating access time
-    $self->_set_status( STATUS_MODIFIED );          # <-- access time modified above
+
+    # We update the atime by default, but if this (otherwise undocoumented)
+    # parameter is explicitly set to false, we'll turn the behavior off
+    if ( ! defined $update_atime ) {
+        $self->{_DATA}->{_SESSION_ATIME} = time();      # <-- updating access time
+        $self->_set_status( STATUS_MODIFIED );          # <-- access time modified above
+    }
+    
     return $self;
 }
 
+
+# set the input as a query object or session ID, depending on what it looks like.  
+sub _set_query_or_sid {
+    my $self = shift;
+    my $query_or_sid = shift;
+    if ( ref $query_or_sid){ $self->{_QUERY}       = $query_or_sid  }
+    else                   { $self->{_CLAIMED_ID}  = $query_or_sid  }
+}
+
+
+sub _load_pluggables {
+    my ($self) = @_;
+
+    my %DEFAULT_FOR = (
+                       driver     => "file",
+                       serializer => "default",
+                       id         => "md5",
+                       );
+    my %SUBDIR_FOR  = (
+                       driver     => "Driver",
+                       serializer => "Serialize",
+                       id         => "ID",
+                       );
+    my $dsn = $self->{_DSN};
+    foreach my $plug qw(driver serializer id) {
+        my $mod_name = $dsn->{ $plug };
+        if (not defined $mod_name) {
+            $mod_name = $DEFAULT_FOR{ $plug };
+        }
+        if ($mod_name =~ /^(\w+)$/) {
+
+            # Looks good.  Put it into the dsn hash
+            $dsn->{ $plug } = $mod_name = $1;
+
+            # Put together the actual module name to load
+            my $prefix = join '::', (__PACKAGE__, $SUBDIR_FOR{ $plug }, q{});
+            $mod_name = $prefix . $mod_name;
+
+            ## See if we can load load it
+            eval "require $mod_name";
+            if ($@) {
+                my $msg = $@;
+                return $self->set_error("couldn't load $mod_name: " . $msg);
+            }
+        }
+        else {
+            # do something here about bad name for a pluggable
+        }
+    }
+    return;
+}
+
 =pod
 
-=item id()
+=head2 id()
 
 Returns effective ID for a session. Since effective ID and claimed ID can differ, valid session id should always
 be retrieved using this method.
 
-=item param($name)
+=head2 param($name)
 
-=item param(-name=E<gt>$name)
+=head2 param(-name=E<gt>$name)
 
 Used in either of the above syntax returns a session parameter set to $name or undef if it doesn't exist. If it's called on a deleted method param() will issue a warning but return value is not defined.
 
-=item param($name, $value)
+=head2 param($name, $value)
 
-=item param(-name=E<gt>$name, -value=E<gt>$value)
+=head2 param(-name=E<gt>$name, -value=E<gt>$value)
 
 Used in either of the above syntax assigns a new value to $name parameter,
 which can later be retrieved with previously introduced param() syntax. C<$value>
@@ -748,11 +905,11 @@ may be a scalar, arrayref or hashref.
 Attempts to set parameter names that start with I<_SESSION_> will trigger
 a warning and undef will be returned.
 
-=item param_hashref()
+=head2 param_hashref()
 
 B<Deprecated>. Use L<dataref()|/"dataref"> instead.
 
-=item dataref()
+=head2 dataref()
 
 Returns reference to session's data table:
 
@@ -763,51 +920,65 @@ Returns reference to session's data table:
 
 Useful for having all session data in a hashref, but too risky to update.
 
-=item save_param()
+=head2 save_param()
 
-=item save_param($query)
+=head2 save_param($query)
 
-=item save_param($query, \@list)
+=head2 save_param($query, \@list)
 
 Saves query parameters to session object. In other words, it's the same as calling L<param($name, $value)|/"param"> for every single query parameter returned by C<< $query->param() >>. The first argument, if present, should be either CGI object or any object which can provide param() method. If it's undef, defaults to the return value of L<query()|/"query">, which returns C<< CGI->new >>. If second argument is present and is a reference to an array, only those query parameters found in the array will be stored in the session. undef is a valid placeholder for any argument to force default behavior.
 
-=item load_param()
+=head2 load_param()
 
-=item load_param($query)
+=head2 load_param($query)
 
-=item load_param($query, \@list)
+=head2 load_param($query, \@list)
 
 Loads session parameters into a query object. The first argument, if present, should be query object, or any other object which can provide param() method. If second argument is present and is a reference to an array, only parameters found in that array will be loaded to the query object.
 
-=item clear()
+=head2 clear()
 
-=item clear('field')
+=head2 clear('field')
 
-=item clear(\@list)
+=head2 clear(\@list)
 
 Clears parameters from the session object.
 
 With no parameters, all fields are cleared. If passed a single parameter or a
 reference to an array, only the named parameters are cleared.
 
-=item flush()
+=head2 flush()
 
-Synchronizes data in the buffer with its copy in disk. Normally it will be called for you just before the program terminates, or session object goes out of scope, so you should never have to flush() on your own.
+Synchronizes data in memory  with the copy serialized by the driver. Call flush() 
+if you need to access the session from outside the current session object. You should
+at least call flush() before your program exits. 
 
-=item atime()
+As a last resort, CGI::Session will automatically call flush for you just
+before the program terminates or session object goes out of scope. This automatic
+behavior was the recommended behavior until the 4.x series. Automatic flushing
+has since proven to be unreliable, and in some cases is now required in places
+that worked with 3.x. For further details see:
+
+ http://rt.cpan.org/Ticket/Display.html?id=17541
+ http://rt.cpan.org/Ticket/Display.html?id=17299
+
+Consequently, always explicitly calling C<flush()> on the session before the program exits
+should be regarded as mandatory until this problem is rectified.
+
+=head2 atime()
 
 Read-only method. Returns the last access time of the session in seconds from epoch. This time is used internally while
 auto-expiring sessions and/or session parameters.
 
-=item ctime()
+=head2 ctime()
 
 Read-only method. Returns the time when the session was first created in seconds from epoch.
 
-=item expire()
+=head2 expire()
 
-=item expire($time)
+=head2 expire($time)
 
-=item expire($param, $time)
+=head2 expire($param, $time)
 
 Sets expiration interval relative to L<atime()|/"atime">.
 
@@ -815,7 +986,10 @@ If used with no arguments, returns the expiration interval if it was ever set. I
 
 Second form sets an expiration time. This value is checked when previously stored session is asked to be retrieved, and if its expiration interval has passed, it will be expunged from the disk immediately. Passing 0 cancels expiration.
 
-By using the third syntax you can set the expiration interval for a particular session parameter, say I<~logged-in>. This would cause the library call clear() on the parameter when its time is up. Passing 0 cancels expiration.
+By using the third syntax you can set the expiration interval for a particular
+session parameter, say I<~logged-in>. This would cause the library call clear()
+on the parameter when its time is up. Note it only makes sense to set this value to 
+something I<earlier> than when the whole session expires.  Passing 0 cancels expiration.
 
 All the time values should be given in the form of seconds. Following keywords are also supported for your convenience:
 
@@ -918,11 +1092,11 @@ sub _str2seconds {
 
 =pod
 
-=item is_new()
+=head2 is_new()
 
 Returns true only for a brand new session.
 
-=item is_expired()
+=head2 is_expired()
 
 Tests whether session initialized using L<load()|/"load"> is to be expired. This method works only on sessions initialized with load():
 
@@ -935,7 +1109,7 @@ Tests whether session initialized using L<load()|/"load"> is to be expired. This
     }
 
 
-=item is_empty()
+=head2 is_empty()
 
 Returns true for sessions that are empty. It's preferred way of testing whether requested session was loaded successfully or not:
 
@@ -950,18 +1124,25 @@ Actually, the above code is nothing but waste. The same effect could've been ach
 
 L<is_empty()|/"is_empty"> is useful only if you wanted to catch requests for expired sessions, and create new session afterwards. See L<is_expired()|/"is_expired"> for an example.
 
-=item delete()
+=head2 delete()
 
-Deletes a session from the data store and empties session data from memory, completely, so subsequent read/write requests on the same object will fail. Technically speaking, it will only set object's status to I<STATUS_DELETED> and will trigger L<flush()|/"flush">, and flush() will do the actual removal.
+Deletes a session from the data store and empties session data from memory, completely, so subsequent read/write requests on the same object will fail. Technically speaking, though, it will only set the object's status to I<STATUS_DELETED>.
 
-=item find( \&code )
+The intention is that in due course (of the program's execution) this will trigger L<flush()|/"flush">, and flush() will do the actual removal.
 
-=item find( $dsn, \&code )
+However: Auto-flushing can be unreliable, and always explicitly calling C<flush()> on the session before the program exits
+should be regarded as mandatory until this problem is rectified.
 
-=item find( $dsn, \&code, \%dsn_args )
+=head2 find( \&code )
+
+=head2 find( $dsn, \&code )
+
+=head2 find( $dsn, \&code, \%dsn_args )
 
 Experimental feature. Executes \&code for every session object stored in disk, passing initialized CGI::Session object as the first argument of \&code. Useful for housekeeping purposes, such as for removing expired sessions. Following line, for instance, will remove sessions already expired, but are still in disk:
 
+The following line, for instance, will remove sessions already expired, but which are still on disk:
+
     CGI::Session->find( sub {} );
 
 Notice, above \&code didn't have to do anything, because load(), which is called to initialize sessions inside find(), will automatically remove expired sessions. Following example will remove all the objects that are 10+ days old:
@@ -969,21 +1150,84 @@ Notice, above \&code didn't have to do anything, because load(), which is called
     CGI::Session->find( \&purge );
     sub purge {
         my ($session) = @_;
-        next if $session->empty;    # <-- already expired?!
+        next if $session->is_empty;    # <-- already expired?!
         if ( ($session->ctime + 3600*240) <= time() ) {
             $session->delete() or warn "couldn't remove " . $session->id . ": " . $session->errstr;
         }
     }
 
-B<Note:> find() is meant to be convenient, not necessarily efficient. It's best suited in cron scripts.
+B<Note>: find will not change the modification or access times on the sessions it returns.
 
-=back
-
-=head1 MISCELLANEOUS METHODS
+Explanation of the 3 parameters to C<find()>:
 
 =over 4
 
-=item remote_addr()
+=item $dsn
+
+This is the DSN (Data Source Name) used by CGI::Session to control what type of
+sessions you previously created and what type of sessions you now wish method
+C<find()> to pass to your callback.
+
+The default value is defined above, in the docs for method C<new()>, and is
+'driver:file;serializer:default;id:md5'.
+
+Do not confuse this DSN with the DSN arguments mentioned just below, under \%dsn_args.
+
+=item \&code
+
+This is the callback provided by you (i.e. the caller of method C<find()>)
+which is called by CGI::Session once for each session found by method C<find()>
+which matches the given $dsn.
+
+There is no default value for this coderef.
+
+When your callback is actually called, the only parameter is a session. If you
+want to call a subroutine you already have with more parameters, you can
+achieve this by creating an anonymous subroutine that calls your subroutine
+with the parameters you want. For example:
+
+    CGI::Session->find($dsn, sub { my_subroutine( @_, 'param 1', 'param 2' ) } );
+    CGI::Session->find($dsn, sub { $coderef->( @_, $extra_arg ) } );
+    
+Or if you wish, you can define a sub generator as such:
+
+    sub coderef_with_args {
+        my ( $coderef, @params ) = @_;
+        return sub { $coderef->( @_, @params ) };
+    }
+    
+    CGI::Session->find($dsn, coderef_with_args( $coderef, 'param 1', 'param 2' ) );
+
+=item \%dsn_args
+
+If your $dsn uses file-based storage, then this hashref might contain keys such as:
+
+    {
+        Directory => Value 1,
+        NoFlock   => Value 2,
+        UMask     => Value 3
+    }
+
+If your $dsn uses db-based storage, then this hashref contains (up to) 3 keys, and looks like:
+
+    {
+        DataSource => Value 1,
+        User       => Value 2,
+        Password   => Value 3
+    }
+
+These 3 form the DSN, username and password used by DBI to control access to your database server,
+and hence are only relevant when using db-based sessions.
+
+The default value of this hashref is undef.
+
+=back
+
+B<Note:> find() is meant to be convenient, not necessarily efficient. It's best suited in cron scripts.
+
+=head1 MISCELLANEOUS METHODS
+
+=head2 remote_addr()
 
 Returns the remote address of the user who created the session for the first time. Returns undef if variable REMOTE_ADDR wasn't present in the environment when the session was created.
 
@@ -993,15 +1237,15 @@ sub remote_addr {   return $_[0]->{_DATA}->{_SESSION_REMOTE_ADDR}   }
 
 =pod
 
-=item errstr()
+=head2 errstr()
 
 Class method. Returns last error message from the library.
 
-=item dump()
+=head2 dump()
 
 Returns a dump of the session object. Useful for debugging purposes only.
 
-=item header()
+=head2 header()
 
 Replacement for L<CGI.pm|CGI>'s header() method. Without this method, you usually need to create a CGI::Cookie object and send it as part of the HTTP header:
 
@@ -1019,25 +1263,19 @@ It will retrieve the name of the session cookie from C<$session->name()> which d
 
 Now, $session->header() uses "MY_SID" as a name for the session cookie.
 
-=item query()
+=head2 query()
 
 Returns query object associated with current session object. Default query object class is L<CGI.pm|CGI>.
 
-=back
-
 =head2 DEPRECATED METHODS
 
 These methods exist solely for for compatibility with CGI::Session 3.x.
 
-=over 4
-
-=item close()
+=head3 close()
 
 Closes the session. Using flush() is recommended instead, since that's exactly what a call
 to close() does now.
 
-=back 
-
 =head1 DISTRIBUTION
 
 CGI::Session consists of several components such as L<drivers|"DRIVERS">, L<serializers|"SERIALIZERS"> and L<id generators|"ID GENERATORS">. This section lists what is available.
@@ -1093,11 +1331,6 @@ Full name: B<CGI::Session::Serialize::freezethaw>
 L<yaml|CGI::Session::Serialize::yaml> - serializes data using YAML. Requires L<YAML> or L<YAML::Syck>.
 Full name: B<CGI::Session::Serialize::yaml>
 
-=item *
-
-L<json|CGI::Session::Serialize::json> - serializes data using JSON. Requires L<JSON::Syck>.
-Full name: B<CGI::Session::Serialize::json>
-
 =back
 
 =head2 ID GENERATORS
@@ -1138,9 +1371,9 @@ CGI::Session evolved to what it is today with the help of following developers.
 
 =item Igor Plisco E<lt>igor@plisco.ruE<gt>
 
-=item Mark Stosberg E<lt>markstos@cpan.orgE<gt>
+=item Mark Stosberg 
 
-=item Matt LeBlanc
+=item Matt LeBlanc E<lt>mleblanc@cpan.orgE<gt>
 
 =item Shawn Sorichetti
 
@@ -1156,11 +1389,11 @@ This library is free software. You can modify and or distribute it under the sam
 You can see what the developers have been up to since the last release by
 checking out the code repository. You can browse the Subversion repository from here:
 
- http://svn.cromedome.net/
+ http://svn.cromedome.net/repos/CGI-Session
 
 Or check it directly with C<svn> from here:
 
- svn://svn.cromedome.net/CGI-Session
+ https://svn.cromedome.net/repos/CGI-Session
 
 =head1 SUPPORT
 
