@@ -1,13 +1,13 @@
 package CGI::Session::Driver::mysql;
 
-# $Id: mysql.pm 257 2006-03-16 07:56:41Z markstos $
+# $Id: mysql.pm 394 2008-03-22 02:35:30Z markstos $
 
 use strict;
 use Carp;
 use CGI::Session::Driver::DBI;
 
 @CGI::Session::Driver::mysql::ISA       = qw( CGI::Session::Driver::DBI );
-$CGI::Session::Driver::mysql::VERSION   = "2.03";
+$CGI::Session::Driver::mysql::VERSION   = '4.30';
 
 sub _mk_dsnstr {
     my ($class, $dsn) = @_;
@@ -47,7 +47,7 @@ sub store {
     croak "store(): usage error" unless $sid && $datastr;
 
     my $dbh = $self->{Handle};
-    $dbh->do("REPLACE INTO " . $self->table_name . " (id, a_session) VALUES(?, ?)", undef, $sid, $datastr)
+    $dbh->do("REPLACE INTO " . $self->table_name . " ($self->{IdColName}, $self->{DataColName}) VALUES(?, ?)", undef, $sid, $datastr)
         or return $self->set_error( "store(): \$dbh->do failed " . $dbh->errstr );
     return 1;
 }
@@ -74,23 +74,51 @@ CGI::Session::Driver::mysql - CGI::Session driver for MySQL database
 
 =head1 SYNOPSIS
 
-    $s = new CGI::Session( "driver:mysql", $sid);
-    $s = new CGI::Session( "driver:mysql", $sid, { DataSource  => 'dbi:mysql:test',
+    $s = new CGI::Session( 'driver:mysql', $sid);
+    $s = new CGI::Session( 'driver:mysql', $sid, { DataSource  => 'dbi:mysql:test',
                                                    User        => 'sherzodr',
                                                    Password    => 'hello' });
-    $s = new CGI::Session( "driver:mysql", $sid, { Handle => $dbh } );
+    $s = new CGI::Session( 'driver:mysql', $sid, { Handle => $dbh } );
 
 =head1 DESCRIPTION
 
 B<mysql> stores session records in a MySQL table. For details see L<CGI::Session::Driver::DBI|CGI::Session::Driver::DBI>, its parent class.
 
+It's especially important for the MySQL driver that the session ID column be
+defined as a primary key, or at least "unique", like this:
+
+ CREATE TABLE sessions (
+     id CHAR(32) NOT NULL PRIMARY KEY,
+     a_session TEXT NOT NULL
+  );
+
+To use different column names, change the 'create table' statement, and then simply do this:
+
+    $s = new CGI::Session('driver:mysql', undef,
+    {
+        TableName=>'session',
+        IdColName=>'my_id',
+        DataColName=>'my_data',
+        DataSource=>'dbi:mysql:project',
+    });
+
+or
+
+    $s = new CGI::Session('driver:mysql', undef,
+    {
+        TableName=>'session',
+        IdColName=>'my_id',
+        DataColName=>'my_data',
+        Handle=>$dbh,
+    });
+
 =head2 DRIVER ARGUMENTS
 
 B<mysql> driver supports all the arguments documented in L<CGI::Session::Driver::DBI|CGI::Session::Driver::DBI>. In addition, I<DataSource> argument can optionally leave leading "dbi:mysql:" string out:
 
-    $s = new CGI::Session( "driver:mysql", $sid, {DataSource=>'shopping_cart'});
+    $s = new CGI::Session( 'driver:mysql', $sid, {DataSource=>'shopping_cart'});
     # is the same as:
-    $s = new CGI::Session( "driver:mysql", $sid, {DataSource=>'dbi:mysql:shopping_cart'});
+    $s = new CGI::Session( 'driver:mysql', $sid, {DataSource=>'dbi:mysql:shopping_cart'});
 
 =head2 BACKWARDS COMPATIBILITY
 
