@@ -1,6 +1,6 @@
 package CGI::Session::Driver::sqlite;
 
-# $Id: sqlite.pm 289 2006-04-01 08:41:52Z markstos $
+# $Id: sqlite.pm 394 2008-03-22 02:35:30Z markstos $
 
 use strict;
 
@@ -9,7 +9,7 @@ use base 'CGI::Session::Driver::DBI';
 use DBI qw(SQL_BLOB);
 use Fcntl;
 
-$CGI::Session::Driver::sqlite::VERSION    = "1.8";
+$CGI::Session::Driver::sqlite::VERSION    = '4.30';
 
 sub init {
     my $self = shift;
@@ -20,7 +20,6 @@ sub init {
 
     $self->SUPER::init() or return;
     
-    $self->{_disconnect} = 1;
     $self->{Handle}->{sqlite_handle_binary_nulls} = 1;
     return 1;
 }
@@ -32,22 +31,39 @@ sub store {
 
     my $dbh = $self->{Handle};
 
-    my $sth = $dbh->prepare("SELECT id FROM " . $self->table_name . " WHERE id=?");
+    my $sth = $dbh->prepare("SELECT $self->{IdColName} FROM " . $self->table_name . " WHERE $self->{IdColName}=?");
     unless ( defined $sth ) {
         return $self->set_error( "store(): \$sth->prepare failed with message " . $dbh->errstr );
     }
 
     $sth->execute( $sid ) or return $self->set_error( "store(): \$sth->execute failed with message " . $dbh->errstr );
     if ( $sth->fetchrow_array ) {
-        __ex_and_ret($dbh,"UPDATE " . $self->table_name . " SET a_session=? WHERE id=?",$datastr,$sid)
+        __ex_and_ret($dbh,"UPDATE " . $self->table_name . " SET $self->{DataColName}=? WHERE $self->{IdColName}=?",$datastr,$sid)
             or return $self->set_error( "store(): serialize to db failed " . $dbh->errstr );
     } else {
-        __ex_and_ret($dbh,"INSERT INTO " . $self->table_name . " (a_session,id) VALUES(?, ?)",$datastr, $sid)
+        __ex_and_ret($dbh,"INSERT INTO " . $self->table_name . " ($self->{DataColName},$self->{IdColName}) VALUES(?, ?)",$datastr, $sid)
             or return $self->set_error( "store(): serialize to db failed " . $dbh->errstr );
     }
     return 1;
 }
 
+sub DESTROY {
+    my $self = shift;
+
+    unless ( $self->{Handle} -> ping ) {
+        $self->set_error(__PACKAGE__ . '::DESTROY(). Database handle has gone away');
+        return;
+	}
+
+    unless ( $self->{Handle}->{AutoCommit} ) {
+        $self->{Handle}->commit;
+    }
+
+    if ( $self->{_disconnect} ) {
+        undef $self->{Handle};
+    }
+}
+
 sub __ex_and_ret {
     my ($dbh,$sql,$datastr,$sid) = @_;
     # fix rt #18183
@@ -76,6 +92,16 @@ CGI::Session::Driver::sqlite - CGI::Session driver for SQLite
     $s = new CGI::Session("driver:sqlite", $sid, {DataSource=>'/my/folder/sessions.sqlt'});
     $s = new CGI::Session("driver:sqlite", $sid, {Handle=>$dbh});
 
+or
+
+    $s = new CGI::Session('driver:sqlite', undef,
+    {
+        TableName=>'session',
+        IdColName=>'my_id',
+        DataColName=>'my_data',
+        Handle=>$dbh,
+    });
+
 =head1 DESCRIPTION
 
 B<sqlite> driver stores session data in SQLite files using L<DBD::SQLite|DBD::SQLite> DBI driver. More details see L<CGI::Session::Driver::DBI|CGI::Session::Driver::DBI>, its parent class.
