@@ -11,7 +11,7 @@ our $CURRENT;
 sub ok_later (&;$);
     
 
-$CGI::Session::Test::Default::VERSION = '1.54';
+$CGI::Session::Test::Default::VERSION = '4.30';
 
 =head1 CGI::Session::Test::Default
 
@@ -87,9 +87,9 @@ sub run {
         ok(1, "=== 1 ===");
         my $session = CGI::Session->load() or die CGI::Session->errstr;
         ok($session, "empty session should be created");
-        ok(!$session->id);
-        ok($session->is_empty);
-        ok(!$session->is_expired);
+        ok(!$session->id, 'Id is empty');
+        ok($session->is_empty, 'Session is empty');
+        ok(!$session->is_expired, 'Session is not expired');
 
         undef $session;
 
@@ -147,7 +147,7 @@ sub run {
             ok($session, "Session was retrieved successfully");
             ok(!$session->is_expired, "session isn't expired yet");
 
-            is($session->id,$sid, "session IDs are consistent: " . $session->id);
+            is($session->id,$sid, "session IDs are consistent");
             ok($session->atime > $session->ctime, "ctime should be older than atime");
             ok(!$session->etime, "etime shouldn't be set yet");
 
